<?php namespace Modules\Forum\Controllers;
/**
 * @website			www.aksaracms.com
 * @copyright		(c) 2021 - Aksara Laboratory
 * @since			version 1.0.0
 */
class Forum extends \Aksara\Laboratory\Core
{
	private $_table									= 'forum__categories';
	
	public function __construct()
	{
		parent::__construct();
		
		$this->set_upload_path('forum');
	}
	
	public function index()
	{
		if($this->_method == 'index')
		{
			$this->select
			('
				(
					SELECT
						COUNT(*)
					FROM
						forum__topics
					WHERE
						forum__topics.category_id = forum__categories.category_id
				) AS topics_count,
				
				(
					SELECT
						COUNT(*)
					FROM
						post__comments
					JOIN
						forum__topics
						ON
							forum__topics.topic_id = post__comments.post_id
					WHERE
						forum__topics.category_id = forum__categories.category_id
				) AS replies_count,
				
				(
					SELECT
						forum__topics.topic_slug
					FROM
						post__comments
					JOIN
						forum__topics
						ON
							forum__topics.topic_id = post__comments.post_id
					WHERE
						forum__topics.category_id = forum__categories.category_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_topic_slug,
				
				(
					SELECT
						forum__topics.topic_title
					FROM
						post__comments
					JOIN
						forum__topics
						ON
							forum__topics.topic_id = post__comments.post_id
					WHERE
						forum__topics.category_id = forum__categories.category_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_topic_title,
				
				(
					SELECT
						MAX(post__comments.timestamp)
					FROM
						post__comments
					JOIN
						forum__topics
						ON
							forum__topics.topic_id = post__comments.post_id
					WHERE
						forum__topics.category_id = forum__categories.category_id
				) AS recent_created_at,
				
				(
					SELECT
						app__users.user_id
					FROM
						post__comments
					JOIN
						app__users
						ON
							app__users.user_id = post__comments.user_id
					JOIN
						forum__topics
						ON
							forum__topics.topic_id = post__comments.post_id
					WHERE
						forum__topics.category_id = forum__categories.category_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_user_id,
				
				(
					SELECT
						app__users.username
					FROM
						post__comments
					JOIN
						app__users
						ON
							app__users.user_id = post__comments.user_id
					JOIN
						forum__topics
						ON
							forum__topics.topic_id = post__comments.post_id
					WHERE
						forum__topics.category_id = forum__categories.category_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_username,
				
				(
					SELECT
						CONCAT(app__users.first_name, \' \', app__users.last_name)
					FROM
						post__comments
					JOIN
						app__users
						ON
							app__users.user_id = post__comments.user_id
					JOIN
						forum__topics
						ON
							forum__topics.topic_id = post__comments.post_id
					WHERE
						forum__topics.category_id = forum__categories.category_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_full_name
			');
		}
		
		$this->set_title(phrase('forum'))
		->set_description(phrase('the_place_to_discuss_something'))
		->set_icon('mdi mdi-sitemap')
		
		->field_order('category_image')
		
		->unset_field('category_id, category_slug')
		
		->set_field
		(
			array
			(
				'category_description'				=> 'textarea',
				'category_image'					=> 'image',
				'status'							=> 'boolean'
			)
		)
		->set_field('category_slug', 'to_slug', 'category_title')
		
		->set_validation
		(
			array
			(
				'category_title'					=> 'required|unique[' . $this->_table . '.category_title.category_id.' . service('request')->getGet('category_id') . ']',
				'category_description'				=> 'required',
				'status'							=> 'boolean'
			)
		)
		
		->set_alias
		(
			array
			(
				'category_title'					=> phrase('category_title'),
				'category_description'				=> phrase('category_description'),
				'status'							=> phrase('status')
			)
		)
		
		->render($this->_table);
	}
}
