<?php namespace Modules\Forum\Controllers;
/**
 * @website			www.aksaracms.com
 * @copyright		(c) 2021 - Aksara Laboratory
 * @since			version 1.0.0
 */
class Topics extends \Aksara\Laboratory\Core
{
	private $_table									= 'forum__topics';
	
	public function __construct()
	{
		parent::__construct();
		
		$this->set_upload_path('forum');
	}
	
	public function index($slug = '')
	{
		if(!$slug)
		{
			$slug									= service('request')->getGet('category');
		}
		
		if(in_array($this->_method, array('create', 'update', 'delete')))
		{
			$slug									= service('request')->getGet('category');
		}
		else
		{
			$this->select
			('
				forum__categories.category_slug,
				
				(
					SELECT
						COUNT(*)
					FROM
						post__comments
					WHERE
						post__comments.post_id = forum__topics.topic_id
				) AS replies_count,
				
				(
					SELECT
						MAX(post__comments.timestamp)
					FROM
						post__comments
					WHERE
						post__comments.post_id = forum__topics.topic_id
				) AS recent_created_at,
				
				(
					SELECT
						app__users.user_id
					FROM
						post__comments
					JOIN
						app__users
						ON
							app__users.user_id = post__comments.user_id
					WHERE
						post__comments.post_id = forum__topics.topic_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_user_id,
				
				(
					SELECT
						app__users.username
					FROM
						post__comments
					JOIN
						app__users
						ON
							app__users.user_id = post__comments.user_id
					WHERE
						post__comments.post_id = forum__topics.topic_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_username,
				
				(
					SELECT
						CONCAT(app__users.first_name, \' \', app__users.last_name)
					FROM
						post__comments
					JOIN
						app__users
						ON
							app__users.user_id = post__comments.user_id
					WHERE
						post__comments.post_id = forum__topics.topic_id
					ORDER BY
						post__comments.timestamp DESC
					LIMIT
						1
				) AS recent_full_name
			')
			
			->join
			(
				'forum__categories',
				'forum__categories.category_id = forum__topics.category_id',
				'LEFT'
			)
			->where
			(
				array
				(
					'forum__categories.category_slug'	=> $slug
				)
			);
		}
		
		if($this->_method == 'create')
		{
			$this->set_default
			(
				array
				(
					'user_id'						=> get_userdata('user_id'),
					'created_at'					=> date('Y-m-d H:i:s'),
					'status'						=> 0
				)
			);
		}
		else if($this->_method == 'update')
		{
			$this->set_default
			(
				array
				(
					'updated_at'					=> date('Y-m-d H:i:s'),
					'status'						=> 0
				)
			)
			->where
			(
				array
				(
					'topic_id'						=> service('request')->getGet('topic_id')
				)
			);
		}
		else if($this->_method == 'delete')
		{
			$this->where
			(
				array
				(
					'topic_id'						=> service('request')->getGet('topic_id')
				)
			);
		}
		
		$category									= $this->model->get_where
		(
			'forum__categories',
			array
			(
				'category_slug'						=> $slug
			),
			1
		)
		->row();
		
		if(!$category)
		{
			return throw_exception(404, phrase('the_category_you_requested_was_not_found'), current_page('../../'));
		}
		
		$this->set_title
		(
			array
			(
				'index'								=> $category->category_title,
				'create'							=> phrase('add_new_topic'),
				'update'							=> phrase('update_topic')
			)
		)
		->set_description($category->category_description)
		->set_icon($category->category_image)
		
		->unset_field('topic_id, category_id, user_id, topic_slug, created_at, updated_at, status')
		
		->set_field
		(
			array
			(
				'topic_description'					=> 'wysiwyg',
				'topic_attachment'					=> 'files'
			)
		)
		->set_field('topic_slug', 'to_slug', 'topic_title')
		
		->set_validation
		(
			array
			(
				'topic_title'						=> 'required|unique[' . $this->_table . '.topic_title.topic_id.' . service('request')->getGet('topic_id') . ']',
				'topic_description'					=> 'required'
			)
		)
		
		->set_alias
		(
			array
			(
				'topic_title'						=> phrase('topic_title'),
				'topic_description'					=> phrase('topic_description'),
				'topic_attachment'					=> phrase('attachment')
			)
		)
		
		->set_default
		(
			array
			(
				'category_id'						=> $category->category_id
			)
		)
		
		->render($this->_table);
	}
	
	public function thread()
	{
		$topic										= $this->model->select
		('
			forum__topics.topic_id,
			forum__topics.category_id,
			forum__topics.topic_title,
			forum__topics.topic_description
		')
		->join
		(
			'forum__categories',
			'forum__categories.category_id = ' . $this->_table . '.category_id'
		)
		->get_where
		(
			$this->_table,
			array
			(
				'forum__categories.category_slug'	=> service('uri')->setSilent()->getSegment(3),
				$this->_table . '.topic_slug'		=> service('uri')->setSilent()->getSegment(4)
			),
			1
		)
		->row();
		
		if(!$topic)
		{
			return throw_exception(404, phrase('the_topic_you_requested_was_not_found'), current_page('../'));
		}
		
		$this->set_title($topic->topic_title)
		->set_description(truncate($topic->topic_description, 160))
		
		->set_output
		(
			array
			(
				'segment'							=> service('uri')->setSilent()->getSegment(3) . '/' . service('uri')->setSilent()->getSegment(4)
			)
		)
		
		->select
		('
			app__users.username,
			app__users.first_name,
			app__users.last_name,
			app__users.photo,
			
			(
				SELECT
					COUNT(*)
				FROM
					post__comments
				WHERE
					post__comments.post_id = forum__topics.topic_id
			) AS replies_count,
			
			(
				SELECT
					COUNT(*)
				FROM
					post__likes
				WHERE
					post_id = ' . $this->_table . '.topic_id
					AND
					post_type = \'forum\'
					AND
					user_id = app__users.user_id
			) AS topic_likes,
		')
		
		->join
		(
			'forum__categories',
			'forum__categories.category_id = ' . $this->_table . '.category_id'
		)
		->join
		(
			'app__users',
			'app__users.user_id = ' . $this->_table . '.user_id'
		)
		
		->where
		(
			array
			(
				'forum__categories.category_id'		=> $topic->category_id,
				$this->_table . '.topic_id'			=> $topic->topic_id
			)
		)
		
		->render($this->_table);
	}
	
	public function report()
	{
		$this->permission->must_ajax(base_url());
		
		if(!in_array(get_userdata('is_logged'), array(1, 2)))
		{
			return throw_exception(403, phrase('unable_to_close_the_tread'));
		}
		
		$query										= $this->model->get_where
		(
			$this->_table,
			array
			(
				'topic_id'							=> (service('request')->getGet('topic_id') ? service('request')->getGet('topic_id') : 0)
			),
			1
		)
		->row();
		
		if(!$query)
		{
			return throw_exception(404, phrase('the_thread_you_would_toreport_does_not_exist'));
		}
		
		if(service('request')->getPost('topic_id') == sha1(service('request')->getGet('topic_id') . ENCRYPTION_KEY . get_userdata('session_generated')))
		{
			$user_id								= get_userdata('user_id');
			
			// update feedback
			$this->model->insert
			(
				'forum__topics_report',
				array
				(
					'topic_id'						=> service('request')->getGet('topic_id'),
					'user_id'						=> $user_id,
					'reason'						=> service('request')->getPost('reason'),
					'timestamp'						=> date('Y-m-d H:i:s')
				)
			);
			
			return throw_exception(200, phrase('the_thread_was_successfully_reported'));
		}
		
		$html										= '
			<form action="' . current_page() . '" method="POST" class="--validate-form">
				<input type="hidden" name="topic_id" value="' . sha1(service('request')->getGet('topic_id') . ENCRYPTION_KEY . get_userdata('session_generated')) . '" />
				<div class="text-center pt-3 pb-3 border-bottom">
					' . phrase('are_you_sure_want_to_report_this_thread') . '
				</div>
				<div class="p-3 border-bottom">
					<div class="form-group">
						<textarea name="reason" class="form-control" id="reason_input" placeholder="' . phrase('write_a_reason') . '" rows="1"></textarea>
					</div>
				</div>
				<div class="p-3">
					<div class="row">
						<div class="col-6">
							<div class="d-grid">
								<button type="button" class="btn btn-light" data-bs-dismiss="modal">
									<i class="mdi mdi-window-close"></i>
									' . phrase('cancel') . '
								</button>
							</div>
						</div>
						<div class="col-6">
							<div class="d-grid">
								<button type="submit" class="btn btn-danger">
									<i class="mdi mdi-check"></i>
									' . phrase('report') . '
								</button>
							</div>
						</div>
					</div>
				</div>
			</form>
		';
		
		return make_json
		(
			array
			(
				'status'							=> 200,
				'meta'								=> array
				(
					'title'							=> phrase('report_the_thread'),
					'icon'							=> 'mdi mdi-alert-outline',
					'popup'							=> true
				),
				'html'								=> $html
			)
		);
	}
	
	public function close()
	{
		$this->permission->must_ajax(base_url());
		
		if(!in_array(get_userdata('is_logged'), array(1, 2)))
		{
			return throw_exception(403, phrase('unable_to_close_the_tread'));
		}
		
		$query										= $this->model->get_where
		(
			$this->_table,
			array
			(
				'topic_id'							=> (service('request')->getGet('topic_id') ? service('request')->getGet('topic_id') : 0)
			),
			1
		)
		->row();
		
		if(!$query)
		{
			return throw_exception(404, phrase('the_thread_you_would_to_close_does_not_exist'));
		}
		
		if(service('request')->getPost('topic_id') == sha1(service('request')->getGet('topic_id') . ENCRYPTION_KEY . get_userdata('session_generated')))
		{
			if($query->status == 2)
			{
				// update feedback
				$this->model->update
				(
					$this->_table,
					array
					(
						'status'					=> 1
					),
					array
					(
						'topic_id'					=> $query->topic_id
					)
				);
				
				return throw_exception(200, phrase('the_thread_was_successfully_marked_as_opened'));
			}
			else
			{
				// update feedback
				$this->model->update
				(
					$this->_table,
					array
					(
						'status'					=> 2
					),
					array
					(
						'topic_id'					=> $query->topic_id
					)
				);
				
				return throw_exception(200, phrase('the_thread_was_successfully_marked_as_closed'));
			}
		}
		
		$html										= '
			<form action="' . current_page() . '" method="POST" class="--validate-form">
				<input type="hidden" name="topic_id" value="' . sha1(service('request')->getGet('topic_id') . ENCRYPTION_KEY . get_userdata('session_generated')) . '" />
				<div class="text-center pt-3 pb-3 border-bottom">
					' . ($query->status != 2 ? phrase('are_you_sure_want_to_close_this_thread') : phrase('are_you_sure_want_to_reopen_this_thread')) . '
				</div>
				<div class="p-3">
					<div class="row">
						<div class="col-6">
							<div class="d-grid">
								<button type="button" class="btn btn-light" data-bs-dismiss="modal">
									<i class="mdi mdi-window-close"></i>
									' . phrase('cancel') . '
								</button>
							</div>
						</div>
						<div class="col-6">
							<div class="d-grid">
								<button type="submit" class="btn btn-danger">
									<i class="mdi mdi-check"></i>
									' . ($query->status != 2 ? phrase('close') : phrase('reopen')) . '
								</button>
							</div>
						</div>
					</div>
				</div>
			</form>
		';
		
		return make_json
		(
			array
			(
				'status'							=> 200,
				'meta'								=> array
				(
					'title'							=> ($query->status != 2 ? phrase('reopen_the_thread') : phrase('close_the_thread')),
					'icon'							=> 'mdi mdi-alert-outline',
					'popup'							=> true
				),
				'html'								=> $html
			)
		);
	}
}
